from typing import List, Optional
from pydantic import BaseModel, Field

from onilock.core.logging_manager import logger


# @TODO : Rename to Account
class Account(BaseModel):
    id: str = Field(description="Password Identification")
    username: str = Field(default="", description="Username")
    encrypted_password: str = Field(description="Encrypted Password")
    url: Optional[str] = Field(default=None, description="URL or Service name")
    description: Optional[str] = Field(default=None, description="Description")
    created_at: int = Field(description="Creation date")


class Profile(BaseModel):
    name: str
    master_password: str = Field(description="Hashed Master Password")
    accounts: List[Account]

    def get_account(self, id: str | int) -> Account | None:
        if isinstance(id, int):
            try:
                return self.accounts[id]
            except IndexError:
                logger.error("Invalid account index")
                return None

        for account in self.accounts:
            if account.id == id:
                return account
        return None

    def remove_account(self, id: str):
        for index, password in enumerate(self.accounts):
            if password.id == id:
                del self.accounts[index]
                break



from datetime import datetime
import os
import logging
import colorlog
from logging import StreamHandler
from logging.handlers import RotatingFileHandler
from typing import Dict
from onilock.core.settings import settings

# @TODO: Change logs directory to /var/log/onilock
LOGS_DIR = "/tmp/logs/onilock"


class LoggingManager:
    def __init__(
        self,
        name: str = "faslava.logging.LoggingManager",
        default_level: int = logging.INFO,
    ):
        """
        Initialize the LoggingManager with a specific logger name.

        Args:
            name (str): Name of the logger.
            default_level (int): Default logging level.
        """
        self.logger = logging.getLogger(name)
        self.logger.setLevel(default_level)
        self.handlers: Dict[str, logging.Handler] = {}

        # @TODO: Change logs directory to /var/log/onilock
        if not os.path.exists(LOGS_DIR):
            os.makedirs(LOGS_DIR)

    def add_console_handler(self, level: int = logging.INFO):
        """Add a console (stdout) logging handler."""
        log_format = "%(log_color)s%(asctime)s %(levelname)s%(reset)s %(message_log_color)s%(name)s%(reset)s %(message)s"

        color_formatter = colorlog.ColoredFormatter(
            log_format,
            log_colors={
                "DEBUG": "cyan",
                "INFO": "green",
                "WARNING": "yellow",
                "ERROR": "red",
                "CRITICAL": "red,bg_white",
            },
            secondary_log_colors={
                "message": {
                    "DEBUG": "light_black",
                    "INFO": "light_black",
                    "WARNING": "light_black",
                    "ERROR": "light_black",
                    "CRITICAL": "light_black",
                },
            },
        )
        handler = StreamHandler()
        handler.setFormatter(color_formatter)
        handler.setLevel(level)
        self.logger.addHandler(handler)
        self.handlers["console"] = handler

    def add_file_handler(
        self,
        filepath: str,
        level: int = logging.INFO,
        max_bytes: int = 10485760,
        backup_count: int = 5,
    ):
        """Add a rotating file logging handler."""
        handler = RotatingFileHandler(
            filepath, maxBytes=max_bytes, backupCount=backup_count
        )
        handler.setLevel(level)
        formatter = logging.Formatter("%(asctime)s %(levelname)s %(name)s %(message)s")
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)
        self.handlers["file"] = handler

    def remove_handler(self, handler_name: str):
        """Remove a handler by name."""
        if handler_name in self.handlers:
            handler = self.handlers.pop(handler_name)
            self.logger.removeHandler(handler)

    def get_logger(self) -> logging.Logger:
        """Return the configured logger."""
        return self.logger


_log_manager = LoggingManager(
    default_level=logging.DEBUG if settings.DEBUG else logging.ERROR
)

# Add console handler
_log_manager.add_console_handler(
    level=logging.DEBUG if settings.DEBUG else logging.ERROR
)

# Add file handler
today = datetime.today().strftime("%Y%m%d")
logging_level = logging.DEBUG if settings.DEBUG else logging.ERROR
_log_manager.add_file_handler(
    filepath=os.path.join(LOGS_DIR, f"{today}.log"),
    level=logging_level,
)


logger = _log_manager.get_logger()



from datetime import datetime
import uuid
import multiprocessing
import os
from typing import Optional
import base64

from cryptography.fernet import Fernet
import pyperclip
import bcrypt
import typer

from onilock.core.settings import settings
from onilock.core.logging_manager import logger
from onilock.core.gpg import (
    delete_pgp_key,
)
from onilock.core.utils import (
    clear_clipboard_after_delay,
    delete_passphrase_keyring,
    delete_secret_key_keyring,
    generate_random_password,
    get_passphrase,
)
from onilock.db import DatabaseManager
from onilock.db.models import Profile, Account


__all__ = [
    "initialize",
    "new_account",
    "copy_account_password",
    "remove_account",
    "delete_profile",
]


def verify_master_password(master_password: str):
    """
    Verify that the provided master password is valid.

    Args:
        id (str): The target password identifier.
        master_password (str): The master password.
    """
    engine = get_profile_engine()
    data = engine.read()
    if not data:
        typer.echo(
            "This database is not initialized. Please use the `init` command to initialize it."
        )
        exit(1)

    profile = Profile(**data)
    hashed_master_password = base64.b64decode(profile.master_password)
    return bcrypt.checkpw(master_password.encode(), hashed_master_password)


def get_profile_engine():
    """Get user config engine."""

    cipher = Fernet(settings.SECRET_KEY.encode())
    db_manager = DatabaseManager(database_url=settings.SETUP_FILEPATH)
    setup_engine = db_manager.get_engine()
    setup_data = setup_engine.read()
    b64encrypted_config_filepath = setup_data[settings.DB_NAME]["filepath"]
    encrypted_filepath = base64.b64decode(b64encrypted_config_filepath)
    config_filepath = cipher.decrypt(encrypted_filepath).decode()
    return db_manager.add_engine("data", config_filepath)


def initialize(master_password: Optional[str] = None, filepath: Optional[str] = None):
    """
    Initialize the password manager whith a master password.

    Note:
        The master password should be very secure and be saved in a safe place.

    Args:
        master_password (Optional[str]): The master password used to secure all the other accounts.
        filepath (Optional[str]): The filepath where the user wants to save their accounts data.
    """
    logger.debug("Initializing database with a master password.")

    name = settings.DB_NAME

    if not filepath:
        filename = str(uuid.uuid5(uuid.NAMESPACE_DNS, os.getlogin()))
        filepath = os.path.join(
            os.path.expanduser("~"), ".onilock", "vault", f"{filename}.oni"
        )

    db_manager = DatabaseManager(database_url=filepath)
    engine = db_manager.get_engine()
    setup_engine = db_manager.add_engine("setup", settings.SETUP_FILEPATH)
    data = engine.read()
    setup_data = setup_engine.read()

    if data or name in setup_data:
        typer.echo("This database is already initialized")
        exit(1)

    if not master_password:
        logger.warning(
            "Master password not provided! A random password will be generated and returned."
        )
        master_password = generate_random_password(
            length=25, include_special_characters=True
        )
        typer.echo(
            f"Generated password: {master_password}\n"
            "This is the only time this password is visible. Make sure you copy it to a safe place before proceding."
        )

    # @TODO: Verify master password strength.

    hashed_master_password = bcrypt.hashpw(master_password.encode(), bcrypt.gensalt())
    b64_hashed_master_password = base64.b64encode(hashed_master_password).decode()

    profile = Profile(
        name=name,
        master_password=b64_hashed_master_password,
        accounts=list(),
    )
    engine.write(profile.model_dump())

    logger.info("Updating the current setup file.")

    # Encrypting filepath
    cipher = Fernet(settings.SECRET_KEY.encode())
    logger.debug("Encrypting filepath.")
    encrypted_filepath = cipher.encrypt(filepath.encode())
    b64_encrypted_filepath = base64.b64encode(encrypted_filepath).decode()

    setup_data[name] = {
        "filepath": b64_encrypted_filepath,
    }

    setup_engine.write(setup_data)

    logger.info("Initialization completed successfully.")
    return master_password


def new_account(
    name: str,
    password: Optional[str] = None,
    username: Optional[str] = None,
    url: Optional[str] = None,
    description: Optional[str] = None,
):
    """
    Register a new account.

    Args:
        name (str): An identifier used to retrieve the password (e.g. github).
        password (Optional[str]): The password to encrypt, automatically generated if not provided.
        username (Optional[str]): The account username
        url (Optional[str]): The url / service where the password is used.
        description (Optional[str]): A password description.
    """
    engine = get_profile_engine()
    data = engine.read()
    if not data:
        typer.echo(
            "This database is not initialized. Please use the `init` command to initialize it."
        )
        exit(1)

    profile = Profile(**data)

    if not password:
        logger.warning("Password not provided, generating it randomly.")
        password = generate_random_password()

    # @TODO: Verify password strength.

    cipher = Fernet(settings.SECRET_KEY.encode())
    logger.debug("Encrypting the password.")
    encrypted_password = cipher.encrypt(password.encode())
    logger.debug(f"Encrypted password: {encrypted_password.decode()}")
    b64_encrypted_password = base64.b64encode(encrypted_password).decode()
    logger.debug(f"B64 Encrypted password: {b64_encrypted_password}")
    password_model = Account(
        id=name,
        encrypted_password=b64_encrypted_password,
        username=username or "",
        url=url,
        description=description,
        created_at=int(datetime.now().timestamp()),
    )
    profile.accounts.append(password_model)
    engine.write(profile.model_dump())
    logger.info("Password saved successfully.")
    return password


def list_accounts():
    """List all available passwords."""

    engine = get_profile_engine()
    data = engine.read()
    profile = Profile(**data)

    typer.echo(f"Accounts list for {profile.name}")

    for index, pwd in enumerate(profile.accounts):
        created_date = datetime.fromtimestamp(pwd.created_at)
        typer.echo(
            f"""
=================== [{index + 1}] {pwd.id} ===================

          username: {pwd.username}
          password: {pwd.encrypted_password[:15]}***{pwd.encrypted_password[-15:]}
               url: {pwd.url}
       description: {pwd.description}
     creation date: {created_date.strftime("%Y-%m-%d %H:%M:%S")}
            """
        )


def copy_account_password(id: str | int):
    """
    Copy the password of the account with the provided ID to the clipboard.

    Args:
        id (str): The target password identifier.
    """
    engine = get_profile_engine()
    data = engine.read()
    if not data:
        typer.echo(
            "This database is not initialized. Please use the `init` command to initialize it."
        )
        exit(1)

    profile = Profile(**data)

    account = profile.get_account(id)
    if not account:
        typer.echo("Invalid account name or index", err=True, color=True)
        exit(1)

    logger.debug(f"Raw password: {account.encrypted_password}")
    logger.debug("Decrypting the password.")
    cipher = Fernet(settings.SECRET_KEY.encode())
    encrypted_password = base64.b64decode(account.encrypted_password)
    decrypted_password = cipher.decrypt(encrypted_password).decode()
    pyperclip.copy(decrypted_password)
    logger.info(f"Password {account.id} copied to clipboard successfully.")
    typer.echo("Password copied to clipboard successfully.")

    logger.debug("Password will be cleared in 25 seconds.")

    process = multiprocessing.Process(
        target=clear_clipboard_after_delay,
        args=(decrypted_password, 10),
    )
    process.start()

    # Immediately exit the main process to allow it to terminate while the child process runs
    os._exit(0)


def remove_account(name: str):
    """
    Remove a password.

    Args:
        name (str): The target account name.
    """
    engine = get_profile_engine()
    data = engine.read()
    if not data:
        typer.echo(
            "This database is not initialized. Please use the `init` command to initialize it."
        )
        exit(1)

    profile = Profile(**data)

    account = profile.get_account(name)

    if not account:
        typer.echo("Invalid account name", err=True, color=True)
        exit(1)

    profile.remove_account(name)
    engine.write(profile.model_dump())


def delete_profile(master_password: str):
    """
    Delete all profile accounts.

    Args:
        master_password (str): Profile master password.
    """
    cipher = Fernet(settings.SECRET_KEY.encode())
    db_manager = DatabaseManager(database_url=settings.SETUP_FILEPATH)
    setup_engine = db_manager.get_engine()
    setup_data = setup_engine.read()
    b64encrypted_config_filepath = setup_data[settings.DB_NAME]["filepath"]
    encrypted_filepath = base64.b64decode(b64encrypted_config_filepath)
    config_filepath = cipher.decrypt(encrypted_filepath).decode()

    master_password_match = verify_master_password(master_password)
    if not master_password_match:
        typer.echo("Invalid master password!")
        exit(1)

    # Delete profile data
    if os.path.exists(config_filepath):
        os.remove(config_filepath)

    # Delete meta data file
    if os.path.exists(settings.SETUP_FILEPATH):
        os.remove(settings.SETUP_FILEPATH)

    # Get passphrase before deleting the keyring
    passphrase = get_passphrase()

    # Delete keyrings
    delete_secret_key_keyring()
    delete_passphrase_keyring()

    # Delete PGP key
    delete_pgp_key(
        passphrase=passphrase,
        gpg_home=settings.GPG_HOME,
        real_name=settings.PGP_REAL_NAME,
    )



import os
import uuid
from enum import Enum
from typing import Optional

from onilock.core.utils import get_passphrase, get_secret_key, str_to_bool


class DBBackEndEnum(Enum):
    JSON = "Json"
    SQLITE = "SQLite"  # Not implemented yet
    POSTGRES = "PostgreSQL"  # Not implemented yet


class Settings:
    """
    A settings class containing the application configuration.
    """

    def __init__(self) -> None:
        try:
            debug = str_to_bool(os.environ.get("ONI_DEBUG", "false"))
            self.DEBUG = debug
        except ValueError:
            pass

        self.SECRET_KEY = os.environ.get("ONI_SECRET_KEY", get_secret_key())
        self.DB_BACKEND = DBBackEndEnum(os.environ.get("ONI_DB_BACKEND", "Json"))
        self.DB_URL = os.environ.get("ONI_DB_URL")
        self.DB_NAME = os.environ.get("ONI_DB_NAME", os.getlogin())
        self.DB_HOST = os.environ.get("ONI_DB_HOST")
        self.DB_USER = os.environ.get("ONI_DB_USER")
        self.DB_PWD = os.environ.get("ONI_DB_PWD")

        self.PASSPHRASE: str = os.environ.get("ONI_GPG_PASSPHRASE", get_passphrase())
        self.GPG_HOME: Optional[str] = os.environ.get("ONI_GPG_HOME", None)
        self.PGP_REAL_NAME: str = os.environ.get(
            "ONI_PGP_REAL_NAME", f"{os.getlogin()}_onilock_pgp"
        )
        self.PGP_EMAIL: str = "pgp@onilock.com"
        self.CHECKSUM_SEPARATOR = "(:|?"

        try:
            db_port = int(os.environ.get("ONI_DB_PORT", "0"))
            self.DB_PORT = db_port
        except ValueError:
            pass

        filename = str(uuid.uuid5(uuid.NAMESPACE_DNS, os.getlogin() + "_oni"))
        self.SETUP_FILEPATH = os.path.join(
            os.path.expanduser("~"),
            ".onilock",
            "vault",
            f"{filename}.oni",
        )


settings = Settings()



from threading import Lock
from typing import Optional

from onilock.core.logging_manager import logger
from onilock.db.engines import EncryptedJsonEngine


def create_engine(database_url: str):
    return EncryptedJsonEngine(db_url=database_url)


class DatabaseManager:
    _instance = None
    _lock = Lock()

    def __new__(cls, **kwargs):
        """Implement thread-safe singleton behavior."""

        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    cls._instance = super().__new__(cls)

        return cls._instance

    def __init__(self, *, database_url: str):
        # Initialize the database engine and session maker only once
        if not getattr(self, "_initialized", False):
            self._engines = {
                "default": create_engine(database_url),
            }
            self._initialized = True
            logger.debug("Database initialized successfully.")

    def get_engine(self, id: Optional[str] = None):
        if id:
            return self._engines[id]

        return self._engines["default"]

    def add_engine(self, id: str, db_url: str):
        if id in self._engines:
            raise Exception(f"Engine with id `{id}` already exists.")

        self._engines = {
            id: create_engine(db_url),
        }
        return self._engines[id]



from typing import Optional
import gnupg


def generate_pgp_key(gpg_home: Optional[str], name: str, email: str, passphrase: str):
    """Generate a new PGP key pair."""
    gpg = gnupg.GPG(gnupghome=gpg_home)
    input_data = gpg.gen_key_input(
        key_type="RSA",
        key_length=4096,  # ALT: 3072
        name_real=name,
        name_email=email,
        passphrase=passphrase,
    )
    return gpg.gen_key(input_data)


def pgp_key_exists(
    gpg_home: Optional[str],
    key_fingerprint: Optional[str] = None,
    key_id: Optional[str] = None,
    real_name: Optional[str] = None,
) -> bool:
    """Check if a key exists in the keyring."""
    gpg = gnupg.GPG(gnupghome=gpg_home)
    keys = gpg.list_keys()

    if key_fingerprint:
        return any(key["fingerprint"] == key_fingerprint for key in keys)

    if key_id:
        # Key IDs are the last 16 characters of the fingerprint
        return any(key_id in key.get("keyid", "") for key in keys)

    if real_name:
        return any(
            real_name == uid.split(" ")[0] for key in keys for uid in key.get("uids")
        )

    return False


def get_pgp_key_info(
    gpg_home: Optional[str] = None,
    real_name: Optional[str] = None,
    key_id: Optional[str] = None,
) -> dict:
    """Get key details if exists in keyring."""
    gpg = gnupg.GPG(gnupghome=gpg_home)
    keys = gpg.list_keys()

    for key in keys:
        uids = key.get("uids", [])

        if real_name:
            if any(real_name == uid.split(" ")[0] for uid in uids):
                return key
        elif key_id:
            if key_id == key.get("keyid"):
                return key

    return None


def delete_pgp_key(
    *,
    passphrase: str,
    gpg_home: Optional[str] = None,
    real_name: Optional[str] = None,
    key_id: Optional[str] = None,
):
    """Delete PGP public and private key."""
    gpg = gnupg.GPG(gnupghome=gpg_home)
    key_info = get_pgp_key_info(gpg_home, real_name, key_id)
    fingerprint = key_info["fingerprint"]

    # Delete the secret key first
    gpg.delete_keys(
        fingerprint,
        secret=True,
        passphrase=passphrase,
    )

    # Then delete the public key
    gpg.delete_keys(fingerprint)



import os
import json
from typing import Any, Dict
import hashlib

import gnupg

from onilock.core.settings import settings
from onilock.core.logging_manager import logger
from onilock.core.gpg import generate_pgp_key, pgp_key_exists


class Engine:
    """Base Database Engine."""

    def __init__(self, db_url: str):
        self.db_url = db_url

    def write(self, data: Any) -> None:
        raise Exception("Unimplimented")

    def read(self) -> Dict:
        raise Exception("Unimplimented")


class JsonEngine(Engine):
    """Json Database Engine."""

    def __init__(self, db_url: str):
        self.filepath = db_url
        return super().__init__(db_url)

    def write(self, data: Dict) -> None:
        parent_dir = os.path.dirname(self.filepath)
        if parent_dir and not os.path.exists(parent_dir):
            logger.debug(f"Parent dir {parent_dir} does not exist. It will be created.")
            os.makedirs(parent_dir)

        with open(self.filepath, "w") as f:
            json.dump(data, f, indent=4)

    def read(self) -> Dict:
        if not os.path.exists(self.filepath):
            return dict()

        with open(self.filepath, "r") as f:
            try:
                return json.load(f)
            except json.JSONDecodeError:
                return dict()


class EncryptedJsonEngine(JsonEngine):
    """PGP-Encrypted JSON Database Engine."""

    def __init__(self, db_url: str):
        super().__init__(db_url)

        passphrase = settings.PASSPHRASE
        gpg_home = settings.GPG_HOME
        email = settings.PGP_EMAIL
        encryption_key = settings.PGP_REAL_NAME

        self.gpg = gnupg.GPG(gnupghome=gpg_home)
        self.encryption_key = encryption_key  # Recipient key fingerprint/ID
        self.passphrase = passphrase  # Passphrase for private key

        key_generated = pgp_key_exists(
            gpg_home=gpg_home,
            real_name=encryption_key,
        )

        if not key_generated:
            generate_pgp_key(
                gpg_home=gpg_home,
                name=encryption_key,
                email=email,
                passphrase=self.passphrase,
            )

    def write(self, data: Dict) -> None:
        """Encrypt data and write to file."""
        parent_dir = os.path.dirname(self.filepath)
        if parent_dir and not os.path.exists(parent_dir):
            os.makedirs(parent_dir)

        # Serialize data to JSON string
        json_data = json.dumps(data, indent=4)

        # Generate checksum
        checksum = hashlib.sha256(json_data.encode()).hexdigest()
        logger.debug(f"Calculated checksum: {checksum}")

        # Encrypt data
        encrypted_data = self.gpg.encrypt(
            f"{checksum}{settings.CHECKSUM_SEPARATOR}{json_data}",
            recipients=self.encryption_key,
            always_trust=True,
            armor=False,  # ← This is crucial to disable base64 encoding
        )

        if not encrypted_data.ok:
            raise RuntimeError(f"Encryption failed: {encrypted_data.status}")

        # Write encrypted data as binary
        with open(self.filepath, "wb") as f:
            f.write(encrypted_data.data)

    def read(self) -> Dict:
        """Read and decrypt data from file."""
        if not os.path.exists(self.filepath):
            return dict()

        with open(self.filepath, "rb") as f:
            encrypted_data = f.read()

            # Decrypt data
            decrypted_data = self.gpg.decrypt(
                encrypted_data, passphrase=self.passphrase
            )

            if not decrypted_data.ok:
                raise RuntimeError(f"Decryption failed: {decrypted_data.status}")

            # Split checksum and data
            try:
                stored_checksum, data = decrypted_data.data.decode().split(
                    settings.CHECKSUM_SEPARATOR, 1
                )
            except ValueError:
                raise ValueError("Invalid file format")

            # Verify file integrity
            current_checksum = hashlib.sha256(data.encode()).hexdigest()
            if current_checksum != stored_checksum:
                raise RuntimeError("Data corruption detected! Checksum mismatch")

            return json.loads(data)



from typing import Optional

import typer

from onilock.core.utils import generate_random_password
from onilock.account_manager import (
    copy_account_password,
    delete_profile,
    initialize,
    list_accounts,
    remove_account,
    new_account,
)

app = typer.Typer()


@app.command()
def init(
    filepath: Optional[str] = None,
    master_password: Optional[str] = None,
):
    """
    Initialize a password manager onilock profile.

    Note:
        The master password should be very secure and be saved in a safe place.

    Args:
        master_password (Optional[str]): The master password used to secure all the other accounts.
        filepath (Optional[str]): The master password used to secure all the other accounts.
    """

    if not master_password:
        typer.echo("\n\nEnter your Master Password:")
        typer.echo(
            "* Ensure that the password is strong and hidden safely.\n"
            "Leave empty to automatically generate a secure master password."
        )
        master_password = typer.prompt("> ", default="", hide_input=True)

    if not filepath:
        typer.echo("\n\nEnter configuration filepath:")
        typer.echo(
            "* Where you want your configuration file stored "
            "(Choose a hidden place to keep stalkers away).\n"
            "Default: ~/.onilock/<auto-generated-name>"
        )
        filepath = typer.prompt("> ", default="")

    # @TODO: Verify filepath integrity and existance.

    return initialize(master_password, filepath)


@app.command()
def new(
    name: str = typer.Option(..., prompt="Enter Account name (e.g. Github)"),
    password: Optional[str] = typer.Option(
        "",
        prompt="Enter Account password.",
        help="If empty, a strong password will be auto-generated.",
        hide_input=True,
    ),
    username: Optional[str] = typer.Option("", prompt="Enter Account username"),
    url: Optional[str] = typer.Option("", prompt="Enter Account URL"),
    description: Optional[str] = typer.Option("", prompt="Enter Account Description"),
):
    """
    Add new account with to onilock.

    Args:
        name (str): Account name.
        password (Optional[str]): The password to encrypt, automatically generated if not provided.
        username (Optional[str]): The account username
        url (Optional[str]): The url / service where the password is used.
        description (Optional[str]): A password description.
    """
    return new_account(name, password, username, url, description)


@app.command("list")
def accounts():
    """List all available accounts."""

    return list_accounts()


@app.command()
def copy(name: str = typer.Option(..., prompt="Account name or index")):
    """
    Copy the password of the account with the provided name or index to the clipboard.

    N.B: You can find the index next to an account's name in the accounts list.

    Args:
        name (str): The target password identifier.
    """
    account_id: str | int = name
    try:
        account_id = int(account_id) - 1
    except ValueError:
        pass
    return copy_account_password(account_id)


@app.command()
def remove(
    name: str = typer.Option(..., prompt="Account name"),
):
    """
    Remove an account.

    Args:
        name (str): The target password identifier.
    """
    return remove_account(name)


@app.command()
def generate(
    len: int = typer.Option(8, prompt="Enter password length"),
    special_chars: bool = typer.Option(True, prompt="Include special characters?"),
):
    """
    Generate and returns a random password
    """
    random_password = generate_random_password(len, special_chars)
    typer.echo(random_password)


@app.command("clear")
def clear_user_data(
    master_password: str = typer.Option(
        prompt="Enter Account's master password.",
        hide_input=True,
    ),
):
    """
    Delete all profile accounts.

    Args:
        master_password (str): Profile master password.
    """
    return delete_profile(master_password)


@app.command()
def version():
    """Print the current version of onilock and exit."""
    with open("pyproject.toml", "r") as f:
        while True:
            line = f.readline()
            if line.startswith("version"):
                version = line.split('"')[1]
                typer.echo(f"OniLock {version}")
                return


if __name__ == "__main__":
    app()



import os
import time
import string
import secrets
import random
import uuid

from cryptography.fernet import Fernet
import keyring
import pyperclip


def get_base_dir():
    return os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))


def clear_clipboard_after_delay(content: str, delay=60):
    """Clears the clipboard after a delay if it still contains the given content."""
    time.sleep(delay)
    cb_content = pyperclip.paste()
    if cb_content == content:  # Check if clipboard still contains the password
        pyperclip.copy("")  # Clear the clipboard


def generate_random_password(
    length: int = 12, include_special_characters: bool = True
) -> str:
    """
    Generate a random and secure password.

    Args:
        length (int): The length of the generated password
        include_special_characters (bool): If False, the password will only contain alpha-numeric characters.

    Returns:
        str : The generated password
    """
    characters = string.ascii_letters + string.digits
    punctuation = "@$!%*?&_}{()-=+"
    password = [
        secrets.choice(string.ascii_lowercase),
        secrets.choice(string.ascii_uppercase),
        secrets.choice(string.digits),
    ]
    if include_special_characters:
        password.append(secrets.choice(punctuation))
        characters += punctuation

    password += [secrets.choice(characters) for _ in range(length)]

    # Shuffle password in-place.
    random.shuffle(password)

    return "".join(password)


def get_secret_key() -> str:
    """
    Retrieve or generate a random secret key to use for the project.
    """

    # Retrieve key securely
    key_name = str(uuid.uuid5(uuid.NAMESPACE_DNS, os.getlogin()))
    stored_key = keyring.get_password("onilock", key_name)
    if stored_key:
        return stored_key

    # Generate and store the key securely
    secret_key = Fernet.generate_key()
    keyring.set_password("onilock", key_name, secret_key.decode())

    return secret_key.decode()


def get_passphrase() -> str:
    """
    Retrieve or generate a random passphrase for the PGP key
    """

    # Retrieve key securely
    key_name = str(uuid.uuid5(uuid.NAMESPACE_DNS, os.getlogin() + "_oni"))
    stored_key = keyring.get_password("onilock", key_name)
    if stored_key:
        return stored_key

    # Generate and store the key securely
    secret_key = generate_random_password(25)
    keyring.set_password("onilock", key_name, secret_key)

    return secret_key


def delete_secret_key_keyring():
    key_name = str(uuid.uuid5(uuid.NAMESPACE_DNS, os.getlogin()))
    keyring.delete_password("onilock", key_name)


def delete_passphrase_keyring():
    key_name = str(uuid.uuid5(uuid.NAMESPACE_DNS, os.getlogin() + "_oni"))
    keyring.delete_password("onilock", key_name)


def str_to_bool(s: str) -> bool:
    """
    Evalueates a strings to either True or False.

    Args:
        s (str): The string to evaluate as a boolean.

    Raises:
        ValueError, if the argument `s` could not be evaluated to a boolean.

    Returns:
        True if the string is in: ("true", "1", "t", "yes", "on")
        True if the string is in: ("false", "0", "f", "no", "off")
    """
    if s.lower() in ("true", "1", "t", "yes", "on"):
        return True
    if s.lower() in ("false", "0", "f", "no", "off"):
        return False
    raise ValueError
